#ifndef DSDRPREPROCESSINGAPI_H 
#define DSDRPREPROCESSINGAPI_H

#include <stdint.h>

#if defined _DOXYGEN || (!defined __GNUC__ && !defined _MSC_VER)
// Doxygen active --> no exports/imports
	#define DSDRPP_DECL
	#define DSDRPP_CONV
#else
	#if defined DSDRPP_DEVELOPMENT
		#if defined __GNUC__
		// Exporting on Linux (no special import option)
			#define DSDRPP_DECL __attribute__((visibility("default")))
			#define DSDRPP_CONV
		#elif defined WIN32 || defined _MSC_VER
		// Export on Windows
			#define DSDRPP_DECL __declspec(dllexport)
			#define DSDRPP_CONV __stdcall
		#endif
	#else
		#if defined WIN32 || defined _MSC_VER
			#define DSDRPP_DEVELOPMENT_DECL __declspec(dllimport)
			#define DSDRPP_CONV __stdcall
		#endif
	#endif

	#ifndef  DSDRPP_DECL
		#define  DSDRPP_DECL
	#endif

	#ifndef  DSDRPP_CONV
		#define  DSDRPP_CONV
	#endif
#endif /* _DOXYGEN */

typedef void* DsDRPPHandle;

typedef enum DsDRPPErrorCode
{   
    DSDRPP_NO_ERROR = 0,                                // No error
    DSDRPP_NULL_POINTER_ERROR = 1,                      // Null pointer
    DSDRPP_NOT_SUPPORTED_PROTOCOL = 2,                  // The protocol is not supported
    DSDRPP_INVALID_MIN_FILL_LEVEL = 3,                  // MinFillLevel should be non-zero and should be less then maxBufferDepth
    DSDRPP_NOT_REACHED_MIN_FILL_LEVEL = 4,              // MinFillLevel condition is not satisfied 
    DSDRPP_INVALID_MAX_BUFFER_DEPTH = 5,                // Max buffer depth should be non-zero. 
    DSDRPP_BUFFER_FULL = 6,                             // MaxBufferDepth is reached.
    DSDRPP_MIN_FILL_LEVEL_EQUAL_MAX_BUFFER_DEPTH = 7,   // MinFillLevel cannot be equal to maxBufferDepth
    DSDRPP_CORRUPTED_DATA = 8,                          // Data is unauthorized changed
    DSDRPP_INCOMPLETE_FRAME = 9,                        // Received frame is not completed and can not be parsed.
}DsDRPPErrorCode;

typedef enum DsDRPPProtocolType
{
    DSDRPP_UNDEFINED_PROTOCOL = 0, // Undefined protocol
    DSDRPP_SOMEIP = 1,             // SomeIp protocol
    DSDRPP_TCP = 2,                // TCP protocol
    DSDRPP_UDP = 3,                // UDP protocol
    DSDRPP_GENERIC = 4,            // Generic protocol
}DsDRPPProtocolType;

typedef struct DsDRPPStatistics
{
    uint64_t numOfTotalFrames; // number of total frames that are received during Parse Buffer method including and dropped frames, reset only when DsDRPP_DestroyHandle is called
    uint64_t numOfProcessedFrames;  // number of processed fragments
    uint64_t numOfProcessedBytes; // number of bytes of processed fragments
    uint64_t numOfFramesInOutputQueue;  // number of frames that are in output queue, frames are filled during InsertFrames() function.
    uint64_t numOfBytesInOutputQueue; // number of bytes that are in output queue, frames are filled during InsertFrames() function. 
    uint64_t numOfFramesReadyToRead; // number of frames that are ready to read, minFillLevel is reached, frames are filled during InsertFrames() function, they can be read using GetFrames() function.
}DsDRPPStatistics;


#ifdef __cplusplus
    extern "C" {
#endif

/*
 @brief Create handle and initialize with minFillLevel, protocolType and maxBufferDepth.

- @param[out] DsDRPPHandle* handle: Pointer to the unique preprocessing library handle.

- @param[in] uint64_t minFillLevel:The minimum number of frames in output buffer level to achieve.

- @param[in] uint64_t maxBufferDepth: The maximum number of frames that can be received.

- @param[in] enum DsDRPPProtocolType protocolType: Protocol identifier that determines which ethernet protocol library handles (e.g. SOMEIP, TCP, UDP)

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).
*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_CreateHandle(DsDRPPHandle* handle, uint64_t minFillLevel, uint64_t maxBufferDepth, DsDRPPProtocolType protocolType);

/*
 @brief Destroy handle. This function releases the memory allocated for a DsDRPPInfo instance previously allocated using DsDRPP_CreateHandle() function. It should be called when the instance is no longer needed to prevent memory leaks.

 -param[in] DsDRPPHandle* handle: Pointer to the unique preprocessing library handle.

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).

*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_DestroyHandle(DsDRPPHandle* handle);

/*
@brief Function resets all internal states, counters and buffers except totalNumOfFrames.

- @param[in] DsDRPPHandle handle: Unique preprocessing library handle.

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).

*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_FlushBuffers(DsDRPPHandle handle);

/*
@brief Function to process received frames from the buffer given by the user, reorder by timestamp or offset, group them together and forward them to output queue. The user must also give total size of bytes which should be read.

- @param[in] DsDRPPHandle handle: Unique preprocessing library handle.

- @param[in] uint8_t* rawData: Pointer to the user buffer from where the frames/data can be read. Frames should be in BusAPI DSSEthReplayData data format.

- @param[in] uint64_t rawDataLength: The total size of the incoming buffer in bytes.

- @param[out] bool* isEndStream: A pointer to a flag indicating whether the end frame of the stream was received.

- @param[out] uint64_t* numOfInsertedBytes - A pointer to the number of inserted bytes into the output buffer.

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).
*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_InsertFrames(DsDRPPHandle handle, uint8_t* rawData, uint64_t rawDataLength, bool* isEndStream, uint64_t* numOfInsertedBytes);

/*
@brief Function will copy frames from output queue to the user buffer. The user should provide the number of bytes which can be copied.

  Frames will be copied if the IsEndStream flag is set or if minFillLevel is reached.

- @param[in] DsDRPPHandle handle: Unique preprocessing library handle.

- @param[in] uint8_t * buffer: Pointer to the user buffer where frames must be copied.

- @param[in] uint64_t bufferSize: Size in bytes of the user buffer where frames must be copied from output queue. Only full frames are written into the buffer.

- @param[in] bool singleFrame: Flag, when set to true, indicates that only one frame must be copied to output buffer.

- @param[out] uint64_t* numOfWrittenBytes - A pointer to the number of bytes written into the user buffer.

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).
*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_GetFrames(DsDRPPHandle handle, uint8_t* buffer, uint64_t bufferSize, bool singleFrame, uint64_t* numOfWrittenBytes);

/*
 @brief  Function to fetch statistics of preprocessed messages.

- @param[in] DsDRPPHandle handle: Unique preprocessing library handle.

- @param[in] DsDRPPStatistics* stats: structure which contains all necessary statistics about preprocessing.

- @return enum DsDRPPErrorCode: Returns one of the defined DRPPAPI Error Codes (e.g. no error, null pointer error ,...).
*/
    DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_GetStatistics(DsDRPPHandle handle, DsDRPPStatistics* stats);

//----------------------------------------------------------------------------------------------------
// DLL: Function pointers

#if (defined DSDRPP_DEVELOPMENT)  || (defined DSETHAPI_MANUALLY_LINK)
  //  DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_CreateHandle(uint64_t minFillLevel, uint64_t maxBufferDepth, DsDRPPProtocolType protocolType, DsDRPPHandle* handle);

    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_CREATE_HANDLE "DsDRPP_CreateHandle"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_INITIALIZE)
    (
        DsDRPPHandle* handle,
        uint64_t minFillLevel,
        uint64_t maxBufferDepth,
        DsDRPPProtocolType protocolType
    );

   // DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_DestroyHandle(DsDRPPHandle* handle);

    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_DESTROY_HANDLE "DsDRPP_DestroyHandle"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_UNINITIALIZE)
    (
        DsDRPPHandle* handle
    );

   // DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_FlushBuffers(DsDRPPHandle handle);
    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_FLUSH_BUFFERS "DsDRPP_FlushBuffers"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_FLUSH)
    (
        DsDRPPHandle handle
    );

    //DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_InsertFrames(DsDRPPHandle handle, uint8_t* rawData, uint64_t rawDataLength, bool* isEndStream, uint64_t* numOfInsertedBytes);
    
    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_INSERT_FRAMES "DsDRPP_InsertFrames"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_INSERT_FRAMES)
    (
        DsDRPPHandle handle,
        uint8_t* rawData,
        uint64_t rawDataLength,
        bool* isEndStream,
        uint64_t* numOfInsertedBytes
    );
    
   // DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_GetFrames(DsDRPPHandle handle, uint8_t * buffer, uint64_t bufferSize, bool singleFrame, uint64_t* numOfWrittenBytes);
    
    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_GET_FRAMES "DsDRPP_GetFrames"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_GET_FRAMES)
    (
        DsDRPPHandle handle,
        uint8_t* rawData,
        uint64_t bufferSize,
        bool singleFrame,
        uint64_t* numOfWrittenBytes
    );

   // DSDRPP_DECL DsDRPPErrorCode DSDRPP_CONV DsDRPP_GetStatistics(DsDRPPHandle handle,  DsDRPPStatistics* stats);

    #define DSDRPP_DEVELOPMENT_FUNC_NAME_PREPROCESSING_GET_STATISTICS "DsDRPP_GetStatistics"
    typedef DsDRPPErrorCode (DSDRPP_CONV *DSDRPP_FUN_PTR_PREPROCESSING_GET_STATISTICS)
    (
        DsDRPPHandle handle,
        DsDRPPStatistics* stats
    );
#endif

#ifdef __cplusplus
    }
#endif


#endif /* DSDRPREPROCESSINGAPI_H */
